@@ -1,4 +1,5 @@
 import pandas as pd
+import numpy as np
 import re
 from logging import Logger
 
@@ -76,13 +77,17 @@ def get_ihc_results(data, log) -> list:
 
         biomarkername = test_results.get("biomarkerName", "")
         result = test_results.get("result", "")
+        result_group = test_results.get("result_group", "")
+
+        if result_group == "No Result":
+            continue
 
         # Grab a field prefix list to hunt for tc / ic fields
         prefix_list = [x[:2] for x in list(test_results)]
         # If ic / tc counts are off, log it and try to load in minimal test info for future troubleshooting
-        if 0 < (prefix_list.count("tc") + prefix_list.count("ic")) < 7:
+        if (prefix_list.count("tc") + prefix_list.count("ic")) not in [0, 3, 7]:
             log.warning(
-                f'IHC test {test_results["biomarkerName"]} has an unexpected pattern: ic/tc field count is {prefix_list.count("tc")+prefix_list.count("ic")}. Should be 0 or 7(PD-L1).'
+                f'IHC test {test_results["biomarkerName"]} has an unexpected pattern: ic/tc field count is {prefix_list.count("tc")+prefix_list.count("ic")}. Should be 0, 3, or 7(PD-L1).'
             )
             trigger = True
             ihc_results.append({"biomarkerName": biomarkername, "result": result})
@@ -90,17 +95,9 @@ def get_ihc_results(data, log) -> list:
         # Check for PD-L1 tests, work through odd handling patterns
         elif test_results["biomarkerName"][:5] == "PD-L1":
             # Tumor cell / immune cell logic
-            if prefix_list.count("tc") == 4 and prefix_list.count("ic") == 3:
-                tc_result = test_results.get("tcResult", "")
-                tc_intensity = test_results.get("tcIntensity", "")
-                tc_stainpercent = test_results.get("tcStainPercent", "")
-                tc_threshold = (
-                    test_results["tcThreshold"].split("or")[-1].strip()
-                    if test_results.get("tcThreshold")
-                    else ""
-                )
+            if prefix_list.count("ic") == 3:
                 ic_result = test_results.get("icResult", "")
-                ic_stainpercent = test_results.get("icStainPercent", "")
+                ic_stainpercent = test_results.get("icStainPercent", np.nan)
 
                 ic_threshold = (
                     test_results["icThreshold"].split("or")[0].strip()
@@ -108,36 +105,69 @@ def get_ihc_results(data, log) -> list:
                     else ""
                 )
 
-                ihc_results.append(
-                    {
-                        "biomarkerName": biomarkername,
-                        "result": result,
-                        "tcResult": tc_result,
-                        "tcIntensity": tc_intensity,
-                        "tcStainPercent": tc_stainpercent,
-                        "tcThreshold": tc_threshold,
-                        "icResult": ic_result,
-                        "icStainPercent": ic_stainpercent,
-                        "icThreshold": ic_threshold,
-                    }
-                )
+                if prefix_list.count("tc") == 4:
+                    tc_result = test_results.get("tcResult", "")
+                    tc_intensity = test_results.get("tcIntensity", "")
+                    tc_stainpercent = test_results.get("tcStainPercent", np.nan)
+                    tc_threshold = (
+                        test_results["tcThreshold"].split("or")[-1].strip()
+                        if test_results.get("tcThreshold")
+                        else ""
+                    )
+                    ihc_results.append(
+                        {
+                            "biomarkerName": biomarkername,
+                            "result": result,
+                            "tcResult": tc_result,
+                            "tcIntensity": tc_intensity,
+                            "tcStainPercent": float(tc_stainpercent),
+                            "tcThreshold": tc_threshold,
+                            "icResult": ic_result,
+                            "icStainPercent": float(ic_stainpercent),
+                            "icThreshold": ic_threshold,
+                        }
+                    )
+                else:
+                    ihc_results.append(
+                        {
+                            "biomarkerName": biomarkername,
+                            "result": result,
+                            "icResult": ic_result,
+                            "icStainPercent": float(ic_stainpercent),
+                            "icThreshold": ic_threshold,
+                        }
+                    )
+
             # PD-L1 (22c3) tests report a cpScore, and no stain or intensity metrics
             elif test_results["biomarkerName"][-6:] == "(22c3)":
-                cpScore = test_results.get("cpScore", "")
-
                 threshold = (
                     test_results["threshold"].split("or")[-1].strip()
                     if test_results.get("threshold")
                     else ""
                 )
-                ihc_results.append(
-                    {
-                        "biomarkerName": biomarkername,
-                        "result": result,
-                        "cpScore": cpScore,
-                        "threshold": threshold,
-                    }
-                )
+
+                tpScore = test_results.get("tpScore", np.nan)
+                if np.isnan(float(tpScore)) == True:
+                    cpScore = test_results.get("cpScore", np.nan)
+
+                    ihc_results.append(
+                        {
+                            "biomarkerName": biomarkername,
+                            "result": result,
+                            "cpScore": float(cpScore),
+                            "threshold": threshold,
+                        }
+                    )
+
+                else:
+                    ihc_results.append(
+                        {
+                            "biomarkerName": biomarkername,
+                            "result": result,
+                            "tpScore": float(tpScore),
+                            "threshold": threshold,
+                        }
+                    )
 
                 # Check if correct pattern is followed
                 # stainPercent and intensity fields should be missing
@@ -148,16 +178,16 @@ def get_ihc_results(data, log) -> list:
                         )
                         trigger = True
                 if test_results.get("stainPercent"):
-                    if test_results["stainPercent"] != "":
+                    if np.isnan(test_results["stainPercent"]) == False:
                         log.warning(
                             f'IHC test {test_results["biomarkerName"]} has an unexpected pattern for "stainPercent": value of "{test_results["stainPercent"]}" was given when None was expected'
                         )
                         trigger = True
 
             # Some PD-L1 tests don't have icThreshold field filled out,
-            # but report IC Threshold in the regualr Threshold field
+            # but report IC Threshold in the regular Threshold field
             elif isinstance((re.search(hidden_ic, test_results["threshold"])), re.Match):
-                stainpercent = test_results.get("stainPercent")
+                stainpercent = test_results.get("stainPercent", np.nan)
 
                 # IC grabs threshold from the front
                 threshold = (
@@ -170,7 +200,7 @@ def get_ihc_results(data, log) -> list:
                     {
                         "biomarkerName": biomarkername,
                         "result": result,
-                        "stainPercent": stainpercent,
+                        "stainPercent": float(stainpercent),
                         "threshold": threshold,
                     }
                 )
@@ -186,8 +216,8 @@ def get_ihc_results(data, log) -> list:
 
             # If no PD-L1 oddities are found, then report PD-L1 as a standard test
             else:
-                intensity = test_results.get("intensity")
-                stainpercent = test_results.get("stainPercent")
+                intensity = test_results.get("intensity", "")
+                stainpercent = test_results.get("stainPercent", np.nan)
                 threshold = (
                     test_results["threshold"].split("or")[-1].strip()
                     if test_results.get("threshold")
@@ -199,13 +229,17 @@ def get_ihc_results(data, log) -> list:
                         "biomarkerName": biomarkername,
                         "result": result,
                         "intensity": intensity,
-                        "stainPercent": stainpercent,
+                        "stainPercent": float(stainpercent),
                         "threshold": threshold,
                     }
                 )
 
-        # Mismatch Repair Status only reports biomarker and result
-        elif test_results["biomarkerName"] == "Mismatch Repair Status":
+        # A growing list of tests only report biomarker and result
+        elif test_results["biomarkerName"] in [
+            "Mismatch Repair Status",
+            "Folfox Responder Similarity",
+            "ER/PR/Her2/Neu",
+        ]:
             ihc_results.append({"biomarkerName": biomarkername, "result": result})
             # Check if correct pattern is followed
             # stainPercent, intensity and threshold fields should be missing
@@ -216,7 +250,7 @@ def get_ihc_results(data, log) -> list:
                     )
                     trigger = True
             if test_results.get("stainPercent"):
-                if test_results["stainPercent"] != "":
+                if np.isnan(test_results["stainPercent"]) == False:
                     log.warning(
                         f'IHC test {test_results["biomarkerName"]} has an unexpected pattern for "stainPercent": value of "{test_results["stainPercent"]}" was given when None was expected'
                     )
@@ -228,10 +262,39 @@ def get_ihc_results(data, log) -> list:
                     )
                     trigger = True
 
+        # Caris has a list of other tests which report result, and stainPercent
+        # [MLH1, MSH2, MSH6, PMS2]
+        elif test_results["biomarkerName"] in ["MLH1", "MSH2", "MSH6", "PMS2"]:
+            stainpercent = test_results.get("stainPercent", np.nan)
+
+            ihc_results.append(
+                {
+                    "biomarkerName": biomarkername,
+                    "result": result,
+                    "stainPercent": float(stainpercent),
+                }
+            )
+
+            # Check if correct pattern is followed
+            # Intensity and threshold fields should be missing
+            if test_results.get("intensity"):
+                if test_results["intensity"] != "":
+                    log.warning(
+                        f'IHC test {test_results["biomarkerName"]} has an unexpected pattern for "intensity": value of "{test_results["intensity"]}" was given when None was expected'
+                    )
+                    trigger = True
+
+            if test_results.get("threshold"):
+                if test_results["threshold"] != "":
+                    log.warning(
+                        f'IHC test {test_results["biomarkerName"]} has an unexpected pattern for "threshold": value of "{test_results["threshold"]}" was given when None was expected'
+                    )
+                    trigger = True
+
         # Standard test reporting
         else:
             intensity = test_results.get("intensity", "")
-            stainpercent = test_results.get("stainPercent", "")
+            stainpercent = test_results.get("stainPercent", np.nan)
             threshold = (
                 test_results["threshold"].split("or")[-1].strip()
                 if test_results.get("threshold")
@@ -243,7 +306,7 @@ def get_ihc_results(data, log) -> list:
                     "biomarkerName": biomarkername,
                     "result": result,
                     "intensity": intensity,
-                    "stainPercent": stainpercent,
+                    "stainPercent": float(stainpercent),
                     "threshold": threshold,
                 }
             )
@@ -254,11 +317,19 @@ def get_ihc_results(data, log) -> list:
     for test in ihc_results:
         for k, v in test.items():
             # Log if any expected fields are left blank
-            if v == "":
-                log.warning(
-                    f'IHC test {test["biomarkerName"]} has an unexpected pattern for field: {k}'
-                )
-                trigger = True
+            if k == "cpScore" or k == "tpScore" or k == "stainPercent":
+                if np.isnan(v):
+                    log.warning(
+                        f'IHC test {test["biomarkerName"]} has an unexpected pattern for field: {k}'
+                    )
+                    trigger = True
+
+            else:
+                if v == "":
+                    log.warning(
+                        f'IHC test {test["biomarkerName"]} has an unexpected pattern for field: {k}'
+                    )
+                    trigger = True
 
     if trigger == False:
         log.info("All IHC tests matched the expected patterns.")
@@ -266,8 +337,15 @@ def get_ihc_results(data, log) -> list:
     return ihc_results
 
 
+def get_test_type(data):
+    if re.search(r"Hybrid_Transcriptome", data):
+        return "MI Profile - Hybrid"
+    else:
+        return "MI Profile"
+
+
 # Build up the manifest iteratively because almost everything is optional
-def extract_metadata(data, prefix, files, infile, ingest_status, log: Logger):
+def extract_metadata(data, prefix, files, source_file_id, ingest_status, log: Logger):
 
     metadata = {}
 
@@ -276,25 +354,23 @@ def extract_metadata(data, prefix, files, infile, ingest_status, log: Logger):
 
     physician_details = get_physician_details(data)
 
-    metadata["testType"] = "MI Profile"
-
-    #  Get the date without the time
-    metadata["indexedDate"] = test_details["receivedDate"].split()[0]
+    metadata["testType"] = get_test_type(str(data))
 
     # Get date of collected and received for the specimen
     metadata["receivedDate"] = get_received_date(specimen_details)
     metadata["collDate"] = get_collected_date(specimen_details)
     metadata["reportDate"] = get_report_date(test_details, log)
 
+    #  Get the date without the time
+    metadata["indexedDate"] = metadata["reportDate"]
+
     patient = data["patientInformation"]
-    metadata["bodySiteSystem"] = "http://carislifesciences.com/bodySite"
+    metadata["bodySiteSystem"] = "http://lifeomic.com/fhir/sequence-body-site"
     metadata["reportID"] = get_report_id(test_details)
     metadata["mrn"] = patient["mrn"]
     metadata["patientLastName"] = patient["lastName"]
 
     metadata["patientDOB"] = patient["dob"]
-    metadata["patientFirstName"] = patient["firstName"]
-    metadata["patientGender"] = patient["gender"].lower()
 
     # Get physician info - ordering name, NPI, and facility
     metadata["medFacilName"] = get_med_facil_name(physician_details)
@@ -302,12 +378,13 @@ def extract_metadata(data, prefix, files, infile, ingest_status, log: Logger):
     metadata["orderingMDName"] = get_ordering_md_name(physician_details)
     metadata["orderingMDNPI"] = get_ordering_md_npi(physician_details)
 
-    metadata["diagnosis"] = patient["diagnosis"]
-    metadata["diagnosisDisplay"] = metadata["diagnosis"]
+    metadata["indicationSystem"] = "http://lifeomic.com/fhir/sequence-indication"
+    metadata["indication"] = patient["diagnosis"]
+    metadata["indicationDisplay"] = metadata["indication"]
 
     metadata["bodySite"] = data["specimenInformation"]["tumorSpecimenInformation"]["specimenSite"]
     metadata["bodySiteDisplay"] = metadata["bodySite"]
-    metadata["sourceFile"] = infile
+    metadata["sourceFileId"] = source_file_id
     pdf = files["pdf"]
     metadata["reportFile"] = f".lifeomic/caris/{prefix}/{pdf}"
 
@@ -316,8 +393,8 @@ def extract_metadata(data, prefix, files, infile, ingest_status, log: Logger):
         {
             "lastName": metadata["patientLastName"],
             "dob": metadata["patientDOB"],
-            "firstName": metadata["patientFirstName"],
-            "gender": metadata["patientGender"],
+            "firstName": patient["firstName"],
+            "gender": patient["gender"].lower(),
             "identifiers": [
                 {
                     "codingCode": "MR",
@@ -330,8 +407,8 @@ def extract_metadata(data, prefix, files, infile, ingest_status, log: Logger):
         else {
             "lastName": metadata["patientLastName"],
             "dob": metadata["patientDOB"],
-            "firstName": metadata["patientFirstName"],
-            "gender": metadata["patientGender"],
+            "firstName": patient["firstName"],
+            "gender": patient["gender"].lower(),
         }
     )
 
@@ -352,6 +429,8 @@ def extract_metadata(data, prefix, files, infile, ingest_status, log: Logger):
     i = 0
     # if not sufficient quantity we won't have test results
     if test_details["reportType"] != "QNS":
+        if isinstance(tests, dict):
+            tests = [tests]
         for test in tests:
             if (
                 "clinical genes" in test["testName"].lower()
@@ -369,7 +448,10 @@ def extract_metadata(data, prefix, files, infile, ingest_status, log: Logger):
                         tmbUnit = info["tumorMutationBurden"]["mutationBurdenUnit"]
                         # Convert from their format, which is "21 per Mb"
                         if tmbUnit == "Mutations/Megabase":
-                            tmbScore = float(tmbScore.split(" per")[0])
+                            if ">150" in tmbScore:
+                                tmbScore = 151.0
+                            else:
+                                tmbScore = float(tmbScore.split(" per")[0])
                         metadata["tmb"] = tmb
                         metadata["tmbScore"] = tmbScore
                     elif "microsatelliteInstability" in info.keys():
@@ -389,6 +471,13 @@ def extract_metadata(data, prefix, files, infile, ingest_status, log: Logger):
                         metadata["lossOfHeterozygosityStatus"] = (
                             "qns" if loh_status == "quality not sufficient" else loh_status
                         )
+                    elif "genomicScarScore" in info.keys():
+                        hrd_status = info["genomicScarScore"]["result"].lower()
+                        hrd_score = info["genomicScarScore"]["score"]
+                        # This comes out as a string, convert to integer for proper ingestion
+                        metadata["hrdStatus"] = (
+                            "qns" if hrd_status == "quality not sufficient" else hrd_status
+                        )
             elif (
                 "CNA" in test["testName"] or "CND" in test["testName"]
             ) and "test_cancellation_reason" not in test.keys():
@@ -423,6 +512,7 @@ def extract_metadata(data, prefix, files, infile, ingest_status, log: Logger):
         "ga4gh.yml",
         "copynumber.csv",
         "structural.csv",
+        "pdf",
     ]
     for ext, filename in files.items():
         if ext not in ingest_files:
