@@ -49,7 +49,7 @@ def process(
 
         base_xml_name = Path(xml_file).stem
 
-        vcf_name = f"{local_output_dir}/{base_xml_name}/{base_xml_name}.tcf"
+        vcf_name = f"{local_output_dir}/{base_xml_name}/{base_xml_name}.modified.vcf"
         vcf_line_count = vcf_etl(vcf_file, vcf_name, base_xml_name, xml_file, log)
         write_cnv = extract_copy_numbers(results_payload_dict, base_xml_name, local_output_dir, log)
         write_fnv = extract_fusion_variant(
